<?php
/**
 * @author Michael Goncharenko	
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package moodle multiauth
 *
 * Authentication Plugin: Pubcookie Authentication
 *
 * Uses Pubcookie Login Server to authenticate users.
 *
 * 2008-03-11  File created.
 */

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
}

require_once($CFG->libdir.'/authlib.php');

/**
 * Manual authentication plugin.
 */
class auth_plugin_pubcookie extends auth_plugin_base {

    /**
     * Constructor.
     */
    function auth_plugin_pubcookie() {
        global $CFG;
        $this->authtype = 'pubcookie';
        $this->config = get_config('auth/pubcookie');


	if(!isset($this->config->pubcookie_installed)) // module's not installed?? Add language fields.
	{
		// add language strings to english lang files

		$langfile = file($CFG->dirroot."/lang/en_utf8/auth.php");
		$f=@fopen($CFG->dirroot."/lang/en_utf8/auth.php", "w");
		if($f)
		{
			foreach($langfile as $lang_string)
			{
				if(strpos($lang_string, "?".">")===0) // looking for closing PHP tag
				{
					fwrite($f, "$"."string['auth_pubcookietitle']=\"Pubcookie authentication\";\n");
					fwrite($f, "$"."string['can_not_write_htaccess']=\"Can not write the auth/pubcookie/.htaccess file: permission denied. Please change the AuthType manually or change the access rights for your web server to be able to write the file.\";\n");
					fwrite($f, "$"."string['auth_pubcookie_authtype']=\"Pubcookie AuthType\";\n");
					fwrite($f, "$"."string['auth_pubcookie_logoutpage']=\"Logout URL (leave empty for default Moodle logout page)\";\n");
					fwrite($f, "$"."string['auth_pubcookie_childplugin']=\"Child authentication plugin\";\n");

					fwrite($f, "$"."string['auth_pubcookiedescription']=\"Pubcookie uses external Single Sign-On server. Please specify the following information:<br />1. Pubcookie AuthType that is used by mod_pubcookie.<br />2. Logout URL that is used by your Pubcookie service. Leave it empty for default Moodle logout function, but remember that logging out from moodle does not log out user from pubcookie service.<br />3. Child authentication plugin. Pubcookie service does not provide us with user information (such as name, surname, email, etc). Child authentication plugin will be used to fetch user information.\";\n");

				}
				fwrite($f, $lang_string);


			}
			
		 fclose($f);
		 set_config('pubcookie_installed',    1,    'auth/pubcookie');
		}
		

	}
        else
        {
         // merge config with child plugin - we would need this in update_user_record() function
         if(isset($this->config->pubcookie_childplugin))
         {  
          // load child plugin and fetch its config
          $childplugin = get_auth_plugin($this->config->pubcookie_childplugin);
          foreach($childplugin->config as $childconfigkey=>$childconfigvalue)
          {
           if(!isset($this->config->{$childconfigkey})) $this->config->{$childconfigkey} = $childconfigvalue;
          }
         }
        }
	
	//print "Pubcookie constructor (".$this->config->pubcookie_authtype.")";
    }

    /**
     * Returns true if the username and password work and false if they are
     * wrong or don't exist.
     *
     * @param string $username The username (with system magic quotes)
     * @param string $password The password (with system magic quotes)
     *
     * @return bool Authentication success or failure.
     */
    function user_login ($username, $password) {

	    $this->config = get_config('auth/pubcookie'); // just in case


	    if(!isset($this->config->pubcookie_installed))
	    {
		    return false;
	    }
	    else
	    {
		    if(isset($_SERVER["REMOTE_USER"]))   
		    { 
			 return true;		   
		    }
		    else {  return false; }
	    }
        /*
	global $CFG;
        if ($user = get_record('user', 'username', $username, 'mnethostid', $CFG->mnet_localhost_id)) {
            return validate_internal_user_password($user, $password);
        }
        return false;
	*/
    }

    /**
     * gets userinfo
     *
     * Called after user is logged in to retrieve his info (name, surname, email, etc).
     * Pubcookie does not provide us with this information, that's why we need to call get_userinfo() function
     *  of the "parent" module.
     *
     * @param  object  $username User name (with system magic quotes)
     * @return object userinfo
     */

    function get_userinfo($username)
    {
	    $config = get_config("auth/pubcookie");
	    if(isset($config->pubcookie_childplugin))
	    {
		    if(!empty($config->pubcookie_childplugin))
		    {
			    $childplugin = $config->pubcookie_childplugin;
			    $plugin_handle = get_auth_plugin($childplugin);
			    return $plugin_handle->get_userinfo($username);
		    }
	    }
	    return null;
    }


    /**
     * Update a local user record from an external source.
     * This is a lighter version of the one in moodlelib -- won't do
     * expensive ops such as enrolment.
     *
     * If you don't pass $updatekeys, there is a performance hit and
     * values removed from LDAP won't be removed from moodle.
     *
     * @param string $username username (with system magic quotes)
     */
    function update_user_record($username, $updatekeys = false) {

            $config = get_config("auth/pubcookie");
            if(isset($config->pubcookie_childplugin))
            {
                    if(!empty($config->pubcookie_childplugin))
                    {
                            $childplugin = $config->pubcookie_childplugin;
                            $plugin_handle = get_auth_plugin($childplugin);
                            return $plugin_handle->update_user_record($username, $updatekeys);
                    }
            }
            return false;
    }


    /**
     * Called when the user record is updated.
     * Modifies user in external database. It takes olduser (before changes) and newuser (after changes)
     * conpares information saved modified information to external db.
     *
     * @param mixed $olduser     Userobject before modifications    (without system magic quotes)
     * @param mixed $newuser     Userobject new modified userobject (without system magic quotes)
     * @return boolean result
     *
     */
    function user_update($olduser, $newuser) {
	    $config = get_config("auth/pubcookie");
	    if(isset($config->pubcookie_childplugin))
	    {
		    if(!empty($config->pubcookie_childplugin))
		    {
			    $childplugin = $config->pubcookie_childplugin;
			    $plugin_handle = get_auth_plugin($childplugin);
			    return $plugin_handle->user_update($olduser, $newuser);
		    }
	    }
	    return true;
    }

    /**
     * Updates local user data
     *
     * called after user authenticates. Loads another plugin and gets userdata from external storage.
     * We need this because pubcookie does not provide us with user info (name, surname, email, etc).
     *
     * @param  object  $user     User table object  (with system magic quotes)
     * @param  string  $username User name (with system magic quotes)
     * @param  string  $password Plaintext password (with system magic quotes)
     * @return boolean result
     *
     */
    function user_authenticated_hook($user, $username, $password)
    {
	    //global $USER;
	    /*
	    global $SESSION;
	    $childplugin = "ldap";	    

	    $username = trim(moodle_strtolower($username)); /// just in case check text case

	    $userauth = get_auth_plugin($childplugin);

	    if ($newinfo = $userauth->get_userinfo($username)) {
		    $newinfo = truncate_userinfo($newinfo);
		    foreach ($newinfo as $key => $value){
			    $confkey = 'field_updatelocal_' . $key;
			    if (!empty($userauth->config->$confkey) and $userauth->config->$confkey === 'onlogin') {
				    $value = addslashes(stripslashes($value));   // Just in case
				    set_field('user', $key, $value, 'username', $username)
				    or error_log("Error updating $key for $username");
			    }
		    }
	    }

	    $user = get_complete_user_data('username', $username);
	    //$USER = $user;
	    $SESSION->user = $user;

	    //var_dump($user);
	    //print "Dead...";
	    // ---- we can not use this function, it ignores $userauth parameter ------ $uzer = update_user_record($username, get_auth_plugin($childplugin));
	    //die;
	    */
    } 


    

    /**
     * Updates the user's password.
     *
     * called when the user password is updated.
     * always returns false. We can not change pubcookie passwords.
     *
     * @param  object  $user        User table object  (with system magic quotes)
     * @param  string  $newpassword Plaintext password (with system magic quotes)
     * @return boolean result
     *
     */
    function user_update_password($user, $newpassword) {
	return false;
    }

    /**
     * Returns true if this authentication plugin is 'internal'.
     *
     * @return bool
     */
    function is_internal() {
        return false;
    }

    /**
     * Returns true if this authentication plugin can change the user's
     * password.
     *
     * @return bool
     */
    function can_change_password() {
        return false;
    }

    /**
     * Returns the URL for changing the user's pw, or empty if the default can
     * be used.
     *
     * @return string
     */
    function change_password_url() {
        return '';
    }

    /**
     * Returns true if plugin allows resetting of internal password.
     *
     * @return bool
     */
    function can_reset_password() {
        return false;
    }

    /**
     * Prints a form for configuring this authentication plugin.
     *
     * This function is called from admin/auth.php, and outputs a full page with
     * a form for configuring this plugin.
     *
     * @param array $page An object containing all the data for this page.
     */
    function config_form($config, $err, $user_fields) {
        include 'config.html';
    }

    /**
     * Processes and stores configuration data for this authentication plugin.
     */
    function process_config($config) {
	    global $CFG, $err;

	    if (!isset ($config->pubcookie_authtype)) {
		    if(isset($_SERVER["AUTH_TYPE"]))
		     $config->pubcookie_authtype = $_SERVER["AUTH_TYPE"];
		    else
		     $config->pubcookie_authtype = 'EGNetID';
	    }
	    set_config('pubcookie_logoutpage',  $config->pubcookie_logoutpage,    'auth/pubcookie');
	    set_config('pubcookie_childplugin', $config->pubcookie_childplugin,    'auth/pubcookie');

	    $f = @fopen($CFG->dirroot."/auth/pubcookie/.htaccess", "w");
	    if($f)
	    {
		    fwrite($f, "AuthType ".$config->pubcookie_authtype."\nrequire valid-user");
		    fclose($f);

		    set_config('pubcookie_authtype',    $config->pubcookie_authtype,    'auth/pubcookie');
	    }
	    else
	    {
		    $err['pubcookie_authtype'] = get_string("can_not_write_htaccess", "auth");
		    return false;
	    }


        return true;
    }


    /**
     * logs out of pubcookie, not only moodle
     */
    function logoutpage_hook()
    {
     global $redirect; // it is used in logout.php
     $config = get_config("auth/pubcookie");

     if(isset($config->pubcookie_logoutpage))     
      if(!empty($config->pubcookie_logoutpage))
      {
	      $redirect = $config->pubcookie_logoutpage;
      }

     return true; // juse in case 
    } 

    
/* ------- example ----------
    function process_config($config) {
	    // set to defaults if undefined
	    if (!isset ($config->host)) {
		    $config->host = '127.0.0.1';
	    }
	    if (!isset ($config->nasport)) {
		    $config->nasport = '1812';
	    }
	    if (!isset ($config->secret)) {
		    $config->secret = '';
	    }
	    if (!isset($config->changepasswordurl)) {
		    $config->changepasswordurl = '';
	    }

	    // save settings
	    set_config('host',    $config->host,    'auth/radius');
	    set_config('nasport', $config->nasport, 'auth/radius');
	    set_config('secret',  $config->secret,  'auth/radius');
	    set_config('changepasswordurl', $config->changepasswordurl, 'auth/radius');

	    return true;
    }
*/																											
    

}

?>
